/*
 * Project: Image
 * File: Viewer.java
 * Package: imageviewer.rgb
 *
 * Created on Jul 19, 2007 10:20:48 PM
 * By Carlos Fernando Bella Cruz
 */
package imageviewer.rgb;

import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Generate and show the image
 */
public class Viewer extends Frame {

    private static final long serialVersionUID = 1L;

    private Generator graphGenerator;

    /**
     * Constructor
     */
    Viewer() {

        super();

        Color[][] color = Reader.read();

        graphGenerator = new Generator(color);

        addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent e) {

                getGraphGenerator().saveImage(Configuration.OUTPUT_FILE);
                System.exit(1);
            }

        });

        addKeyListener(new KeyAdapter() {

            public void keyPressed(KeyEvent e) {

                repaint();

            }
        });

    }

    /**
     * Retrieve the graph generator
     * @return
     */
    public Generator getGraphGenerator() {

        return graphGenerator;
    }

    /**
     * Instantiate the viewer
     * @param args
     */
    public static void main(String[] args) {

        final Viewer graph = new Viewer();

        graph.setSize(Configuration.X + Configuration.WindowAddition_X,
            Configuration.Y + Configuration.WindowAddition_Y);

        graph.setVisible(true);
    }

    /* (non-Javadoc)
     * @see java.awt.Container#paint(java.awt.Graphics)
     */
    public void paint(Graphics g) {

        super.paint(g);

        Insets insets = getInsets();

        g.translate(insets.left, insets.top);

        g.drawImage(graphGenerator.getImage(), 0, 0, null);

    }

    /* (non-Javadoc)
     * @see java.awt.Container#update(java.awt.Graphics)
     */
    public void update(Graphics g) {

        paint(g);

    }
}
