/* definic.h - definicoes gerais para a gramatica do scanline */

#include <stdio.h>
#include <string.h>

#define ARQUIVO_SAIDA "scanline.in"

#define NODOINICIO "CENA"
#define MAXSTRING 30

#define ARQ_LIXO "lixo"

#define VERDADEIRO 1
#define FALSO      0

#define EXIT_FAILURE 1

/***************************************/
/* Variaveis para a tabela de simbolos */
/***************************************/

/* definicao da lista de luzes */

#define S_SOL     1
#define S_PONTUAL 2
#define S_SPOT    3

typedef struct nodo_luz *apont_luz;

typedef struct nodo_luz {
  int tipo_luz;
  char nome [MAXSTRING];
  float kd, ks, ka,      /* Coeficientes de cores da luz */
        posicao [3],     /* Posicao da luz */
        direcao [3],     /* Direcao da luz no caso de luz SPOT e SOL */
        abertura;        /* Angulo de agrangencia da luz SPOT somente */
  apont_luz prox_luz;    /* Implementacao da lista de luzes */
};

int num_luzes = 0;       /* Contador de quantas luzes sao usadas */

/* definicao da lista de materiais */

typedef struct nodo_mater *apont_mater;

typedef struct nodo_mater {
  char nome [MAXSTRING];    /* Nome do material */
  float kd [3],             /* Constantes de cores */
        ks [3],
        ka [3],
        const_phong;        /* Contante de phong */
  int   num_objetos;        /* Numero de objetos com este material */
  apont_mater prox_mater;   /* Implementacao da lista de materiais */
};

/* definicao dos nodos do grafo direcionado de objetos */

typedef struct nodo_objeto *apont_objeto;

typedef struct nodo_ligac *apont_ligac;

typedef struct nodo_objeto {
  char nome [MAXSTRING];     /* Nome do objeto */
  apont_mater mater_objeto;  /* Indicacao do material do objeto */
  apont_ligac ligac_objeto;  /* Fila de nodos ligacao p/ indicar nodos filhos */
  apont_objeto prox_objeto;  /* Implementacao da lista de objetos */
};

typedef struct nodo_ligac {
  float transf [4] [4];      /* Transformacao do nodo filho para o nodo pai */
  int nivel_parent;          /* Implementacao da operacao de parenteses */
  apont_objeto objeto_filho; /* Ligacao para o nodo filho */
  apont_ligac prox_ligac;    /* Implement. da lista de ligacao p/ nodos filho */
};

/* definicao dos nodos da lista de objetos definitivos */

typedef struct nodo_definit *apont_definit;

typedef struct nodo_definit {
  char nome [MAXSTRING];
  float transf [4] [4];
  apont_mater material;
  apont_definit prox_definit;
};

#define TS_PARALELA      0
#define TS_PERSPECTIVA   1

float TS_observador   [3]     = { 0,  0,  1 },
      TS_vertical     [3]     = { 0,  1,  0 },
      TS_direcao      [3]     = { 0,  0, -1 },
      TS_normal       [3]     = { 0,  0, -1 },
      TS_foco                 = 3,
      TS_janela       [4]     = { -1, -1, 1, 1 },
      TS_luz_ambiente [3]     = { 0,  0,  0 },
      TS_base         [3] [3] = { 0, 0, 0,
				  0, 0, 0,
				  0, 0, 0 };
int   TS_projecao         = TS_PERSPECTIVA;

/* Apontadores para o inicio das estruturas de dados */

apont_luz lista_luz = NULL;           /* lista de luzes */
apont_mater lista_mater = NULL;       /* lista dos materiais utilizados */
apont_objeto grafo_objeto = NULL,     /* grafo direcionado de objetos */
             grafo_fim_objeto = NULL; /* ultimo elemento da lista de objetos */
apont_definit lista_definit = NULL;   /* lista dos objetos definitivos */

/************************/
/* Variaveis para o LEX */
/************************/

float L_float;            /* variavel para retornar floats para o parser */
char  L_nome [MAXSTRING]; /* variavel para retornar nomes para o parser */

/*************************/
/* Variaveis para o YACC */
/*************************/

int Y_erros; /* armazenar a quantidade de erros ocorridos na leitura do arq. */

int Y_linha = 1; /* marcador da linha do arquivo fonte */

float Y_ponto_x, /* armazena a passagem das coordenadas de um ponto ou vetor */
      Y_ponto_y,
      Y_ponto_z;

float Y_matriz      [4] [4], /* armazenamento da matriz de transformacao */
      Y_matriz_temp [4] [4]; /* matriz usada para atualizar a matr.transf. */

float Y_material [3] [3]; /* armazenamento dos coeficientes do material */

float Y_phong; /* armazenamento da constante de Phong */

char Y_nome [MAXSTRING]; /* armazenamento de conteudo de um nome */

int estado_parenteses = 0; /* Controla os niveis de parenteses para transfs. */

FILE *saida; /* arquivo de saida */

/* Apontadores para auxiliar temporariamente a construcao do nodo pelo YACC */

apont_luz luz_temp = NULL;
apont_mater mater_temp = NULL;
apont_objeto objeto_temp = NULL, objeto_filho_temp = NULL;
