/* f.c - Arquivo Biblioteca */

#ifdef DOS
#include <alloc.h>
#else
#include <malloc.h>
#define farmalloc malloc
#endif

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "f.h"

/*-------------------------------------------------------------*/
void *aloca (n)
unsigned long int n;
{
  void *p;

  if((p = farmalloc (n)) == NULL)
  {
    printf ("Erro: falta de memoria.\n");
    exit(1);
  }
  return p;
}

/*-------------------------------------------------------------*/
void adicao (u, v, res)
ponto3d *u;
ponto3d *v;
ponto3d *res;
{
  res->x = u->x + v->x;  /* x1 + x2 */
  res->y = u->y + v->y;  /* y1 + y2 */
  res->z = u->z + v->z;  /* z1 + z2 */
}

/*-------------------------------------------------------------*/
void subtracao (u, v, res)
ponto3d *u;
ponto3d *v;
ponto3d *res;
{
  res->x = u->x - v->x;  /* x1 - x2 */
  res->y = u->y - v->y;  /* y1 - y2 */
  res->z = u->z - v->z;  /* z1 - z2 */
}

/*-------------------------------------------------------------*/
void realXvetor (float k, ponto3d *v, ponto3d *res)
{
  /* k(x,y,z) = (kx,ky,kz) */
  res->x = k*v->x;
  res->y = k*v->y;
  res->z = k*v->z;
}

/*-------------------------------------------------------------*/
void normalizacao (v, res)
ponto3d *v;
ponto3d *res;
{
  float m = modulo (v);

  /* n = v/|v| */
  if (m != 0)
  {
    res->x = v->x / m;
    res->y = v->y / m;
    res->z = v->z / m;
  }
  else
    res->x = res->y = res->z = 0.0;
}

/*-------------------------------------------------------------*/
float distancia (u, v)
ponto3d *u;
ponto3d *v;
{
  float x, y, z;

  x = (u->x - v->x) * (u->x - v->x);
  y = (u->y - v->y) * (u->y - v->y);
  z = (u->z - v->z) * (u->z - v->z);

  return sqrt (x + y + z);
}

/*-------------------------------------------------------------*/
void projecao_uv (u, v, res)
ponto3d *u;
ponto3d *v;
ponto3d *res;
{
  float num = produto_escalar (u, v) / produto_escalar (v, v);

  /* proj u = (u*v/v*v)*v */
  /*    v                 */
  res->x = num * v->x;
  res->y = num * v->y;
  res->z = num * v->z;
}

/*-------------------------------------------------------------*/
void produto_vetorial (u, v, res)
ponto3d *u;
ponto3d *v;
ponto3d *res;
{
  ponto3d r;
  
  r.x = u->y * v->z - u->z * v->y;     /* y1*z2 - z1*y2 */
  r.y = -(u->x * v->z - u->z * v->x);  /* x1*z2 - z1*x2 */
  r.z = u->x * v->y - u->y * v->x;     /* x1*y2 - y1*x2 */
  
  res->x = r.x;
  res->y = r.y;
  res->z = r.z;
}

/*-------------------------------------------------------------*/
void vetorXmatriz (v, a, res)
ponto3d *v;
matriz3x3 a;
ponto3d *res;
{
  ponto3d r;
  
  r.x = v->x * a[0][0] + v->y * a[1][0] + v->z * a[2][0];
  r.y = v->x * a[0][1] + v->y * a[1][1] + v->z * a[2][1];
  r.z = v->x * a[0][2] + v->y * a[1][2] + v->z * a[2][2];
 
  res->x = r.x;
  res->y = r.y;
  res->z = r.z;
}

/*-------------------------------------------------------------*/
void vetorXmatriz4x4 (v, a, res)
ponto3d *v;
matriz4x4 a;
ponto3d *res;
{
  ponto3d r;
  
  r.x = v->x * a[0][0] + v->y * a[1][0] + v->z * a[2][0] + a[3][0];
  r.y = v->x * a[0][1] + v->y * a[1][1] + v->z * a[2][1] + a[3][1];
  r.z = v->x * a[0][2] + v->y * a[1][2] + v->z * a[2][2] + a[3][2];

  res->x = r.x;
  res->y = r.y;
  res->z = r.z;
}

/*-------------------------------------------------------------*/
void adicao3x3 (a, b, res)
matriz3x3 a;
matriz3x3 b;
matriz3x3 res;
{
  int i, j;

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      res[i][j] = a[i][j] + b[i][j];
}

/*-------------------------------------------------------------*/
void subtracao3x3 (a, b, res)
matriz3x3 a;
matriz3x3 b;
matriz3x3 res;
{
  int i, j;

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      res[i][j] = a[i][j] - b[i][j];
}

/*-------------------------------------------------------------*/
void multiplicacao3x3 (a, b, res)
matriz3x3 a;
matriz3x3 b;
matriz3x3 res;
{
  int i, j, k;

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      for (k = 0, res[i][j] = 0; k < 3; k++)
        res[i][j] += a[i][k] * b[k][j];
}

/*-------------------------------------------------------------*/
void transposta3x3(a, res)
matriz3x3 a;
matriz3x3 res;
{
  int i, j;

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      res[i][j] = a[j][i];
}

/*-------------------------------------------------------------*/
void adicao4x4 (a, b, res)
matriz4x4 a;
matriz4x4 b;
matriz4x4 res;
{
  int i, j;

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      res[i][j] = a[i][j] + b[i][j];
}

/*-------------------------------------------------------------*/
void subtracao4x4 (a, b, res)
matriz4x4 a;
matriz4x4 b;
matriz4x4 res;
{
  int i, j;

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      res[i][j] = a[i][j] - b[i][j];
}

/*-------------------------------------------------------------*/
void multiplicacao4x4 (a, b, res)
matriz4x4 a;
matriz4x4 b;
matriz4x4 res;
{
  int i, j, k;

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      for (k = 0, res[i][j] = 0; k < 4; k++)
        res[i][j] += a[i][k] * b[k][j];
}

void calcula_intersecao (ponto3d *p, ponto3d *q, float d, ponto3d *r)
{
  float t = - (p->z - d)/(q->z - p->z);
  
  r->x = p->x + (q->x - p->x) * t;  
  r->y = p->y + (q->y - p->y) * t;  
  r->z = d;  
}
