/* malha.c - Gerenciamento da malha na memoria. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define DOS

#ifdef DOS
#include <alloc.h>
#else
#include <malloc.h>
#endif

#include "f.h"
#include "camera.h"
#include "malha.h"
#include "sl.h"

FILE *abre_arquivo(nome, modo)
char *nome;
char *modo;
{
  FILE *arq;

  if ((arq=fopen(nome,modo))==NULL)
  {
    printf("Erro na abertura do arquivo \"%s\".\n", nome);
    exit(1);
  }
  return arq;
}

static char dir_cena[TAM_NOME_ARQUIVO];

void le_luzes(arq_cena, cena)
FILE *arq_cena;
t_cena *cena;
{
  int i;
  t_luz *luz; 

  fscanf(arq_cena,"%u\n", &cena->n_luzes);

  cena->luzes = ALOCA(cena->n_luzes, t_luz);
  
  luz = &cena->luzes[0];

  /* leitura da luz ambiente */
  fscanf(arq_cena,"%f %f %f\n", &luz->intensidade[RED],
                                   &luz->intensidade[GREEN],
                                   &luz->intensidade[BLUE]);

  for(i = 1; i < cena->n_luzes; i++)
  {
    luz = &cena->luzes[i];

    fscanf(arq_cena,"%f %f %f", &luz->intensidade[RED],
                                   &luz->intensidade[GREEN],
                                   &luz->intensidade[BLUE]);
    fscanf(arq_cena,"%u", &luz->tipo);
    switch(luz->tipo)
    { 
      float angulo;
      case LUZ_PONTUAL:
	     fscanf(arq_cena,"%f %f %f\n", &luz->posicao.x,
					 &luz->posicao.y,
					 &luz->posicao.z);
	     break;
      case LUZ_SPOT:
	     fscanf(arq_cena,"%f %f %f", &luz->posicao.x,
				       &luz->posicao.y,
				       &luz->posicao.z);
	     fscanf(arq_cena,"%f %f %f", &luz->direcao.x,
				       &luz->direcao.y,
				       &luz->direcao.z);
	fscanf(arq_cena,"%f\n", &angulo);
	luz->coss = cos (angulo);
        break;
      case LUZ_SOL:
	fscanf(arq_cena,"%f %f %f\n", &luz->direcao.x,
					 &luz->direcao.y,
					 &luz->direcao.z);
        break;
      default:
        printf("Erro: tipo de luz nao reconhecido.\n");
        break;
    }
  }
}

extern unsigned int tamx, tamy;

void le_camera(arq_cena, cena)
FILE *arq_cena;
t_cena *cena;
{
  t_camera *camera = &cena->camera;

  fscanf(arq_cena,"%f %f %f\n", &camera->observador.x,
                                   &camera->observador.y,
                                   &camera->observador.z);
  fscanf(arq_cena,"%f %f %f\n", &camera->direcao.x,
                                   &camera->direcao.y,
                                   &camera->direcao.z);
  fscanf(arq_cena,"%f %f %f\n", &camera->normal.x,
                                   &camera->normal.y,
                                   &camera->normal.z);
  fscanf(arq_cena,"%f %f %f\n", &camera->vertical.x,
                                   &camera->vertical.y,
				   &camera->vertical.z);
  fscanf(arq_cena,"%f\n", &camera->foco);
  fscanf(arq_cena,"%f %f %f %f\n", &camera->esqj, &camera->infj,
                                       &camera->dirj, &camera->supj);
  fscanf(arq_cena,"%u\n", &camera->tipo_proj);

  posicao(&camera->observador);
  direcao(&camera->direcao);
  normal(&camera->normal);
  vertical(&camera->vertical);
  foco(camera->foco);
  tipo_projecao(camera->tipo_proj);
  janela(camera->esqj, camera->infj, camera->dirj, camera->supj);
  tela(0, 0, tamx, tamy);
}

void le_transf(arq_cena, matriz_transf)
FILE *arq_cena;
matriz4x4 matriz_transf;
{
  int i, j;

  for(i = 0; i < 4; i++)
    for(j = 0; j < 4; j++)
      fscanf(arq_cena,"%f", &matriz_transf[i][j]);

  fscanf(arq_cena,"\n");
}

void calcula_normal(m, p, normal)
t_malha *m;
poligono *p;
ponto3d *normal;
{
  ponto3d u, v, n, *a, *b, *c;

  a = &m->vert[p->vert[0]];
  b = &m->vert[p->vert[1]];
  c = &m->vert[p->vert[2]];
  subtracao(a, b, &u);
  subtracao(c, b, &v);
  produto_vetorial(&u, &v, &n);
  normalizacao(&n, normal);
}

void calcula_normais(m)
t_malha *m;
{
  int i;
 
  for (i = 0; i < m->n_pol;  i++)  
  {
    poligono *p = &m->polig[i];
    ponto3d *normal = &m->normal[i];

    calcula_normal(m, p, normal);
  }
}

void remove_poligono (m, ip)
t_malha *m;
int ip;
{
  poligono *p = &m->polig [ip];

  free (p->vert); 
  p->n_vert = m->polig [m->n_pol - 1].n_vert;
  p->vert = m->polig [m->n_pol - 1].vert;
  p->material = m->polig [m->n_pol - 1].material;
  m->normal[ip].x = m->normal[m->n_pol - 1].x;
  m->normal[ip].y = m->normal[m->n_pol - 1].y;
  m->normal[ip].z = m->normal[m->n_pol - 1].z;
  m->n_pol--; 
}


ponto3d *calcula_normal_vertices(m)
t_malha *m;
{
  int i,j;
  ponto3d *normal_vert = ALOCA(m->n_vert, ponto3d);

  m->normal = ALOCA(m->n_pol, ponto3d); 
  calcula_normais(m);

  for(i=0; i < m->n_vert; i++)
	normal_vert[i].x = normal_vert[i].y = normal_vert[i].z = 0.0;

  for(i=0; i < m->n_pol; i++)
  {
    poligono *p = &m->polig[i];

    for(j=0; j < p->n_vert; j++)
    {
	  ponto3d *q = &normal_vert[p->vert[j]];

	  adicao(q , &m->normal[i], q);
    }
  }

  for (i = 0; i < m->n_pol; i++)
     if (escondido (&m->normal[i], &m->vert[m->polig[i].vert[0]])) 
       remove_poligono (m, i--);

  free(m->normal);
  m->normal = ALOCA(m->n_vert, ponto3d);
  for(i=0; i < m->n_vert; i++)
    normalizacao(&normal_vert[i], &m->normal[i]);

  free (normal_vert); 
     
  return m->normal;
}


/*--------------------------------------------------------------*/
void le_objeto(nome_arq_objeto, m, material, matriz_transf,
               offset_vert, offset_polig)
char *nome_arq_objeto;
t_malha *m;
unsigned int material;
matriz4x4 matriz_transf;
unsigned int *offset_vert;
unsigned int *offset_polig;
{
  FILE *arq_objeto;
  unsigned int i, j, n_vert, n_polig;
  char ant[TAM_NOME_ARQUIVO];

  strcpy(ant, dir_cena);
  arq_objeto = abre_arquivo(strcat(strcat(dir_cena,nome_arq_objeto),".pol"), "r");
  strcpy(dir_cena, ant);

  fscanf(arq_objeto,"%u %u\n", &n_vert, &n_polig);

  for(i = 0; i < n_vert; i++)
  {
    ponto3d *vert = &m->vert[i + *offset_vert],
	    aux;

    fscanf(arq_objeto,"%f %f %f\n", &aux.x, &aux.y, &aux.z);
    vetorXmatriz4x4(&aux, matriz_transf, vert);
  }

  for(i = 0; i < n_polig; i++)
  {
    poligono *p = &m->polig[i + *offset_polig];

    p->material = material;
    fscanf(arq_objeto,"\n%u", &p->n_vert);
    p->vert = ALOCA(p->n_vert, unsigned int);
    for(j = 0; j < p->n_vert; j++)
    {
      fscanf(arq_objeto,"%u", &p->vert[j]);
      p->vert[j] += *offset_vert;
    }
  }
  fclose(arq_objeto);
  *offset_vert += n_vert;
  *offset_polig += n_polig;
}

void le_material (arq_cena, mat)
FILE *arq_cena;
vetor_materiais *mat;
{
  fscanf(arq_cena,"%f %f %f %f %f %f %f %f %f %u\n",
          &mat->coeficiente [RED] [KA],
          &mat->coeficiente [GREEN] [KA],
          &mat->coeficiente [BLUE] [KA],
          &mat->coeficiente [RED] [KD],
          &mat->coeficiente [GREEN] [KD],
          &mat->coeficiente [BLUE] [KD],
          &mat->coeficiente [RED] [KS],
          &mat->coeficiente [GREEN] [KS],
          &mat->coeficiente [BLUE] [KS],
          &mat->exp_phong);
}

/*--------------------------------------------------------------*/
void le_malhas(arq_cena, cena)
FILE *arq_cena;
t_cena *cena;
{
  long int pos_arquivo;
  unsigned int i, j, n_malhas,
               n_objetos, n_objetos_total,
               offset_vert, offset_polig,
               ind_mat;
  t_malha *m = &cena->malha;
  matriz4x4 *matriz_transf;
  char ant[TAM_NOME_ARQUIVO];

  fscanf(arq_cena,"%u\n", &n_malhas);
  m->materiais = ALOCA(n_malhas, vetor_materiais);

  pos_arquivo = ftell(arq_cena); /* guarda a posicao corrente do arquivo */

  n_objetos_total = m->n_vert = m->n_pol = 0;

  for(i = 0; i < n_malhas; i++)
  {
    fscanf(arq_cena,"%u\n", &n_objetos);
    n_objetos_total += n_objetos;
    for(j = 0; j < n_objetos; j++)
    {
      char nome_arq_objeto[TAM_NOME_ARQUIVO];
      FILE *arq_objeto;
      unsigned int n_vert, n_polig;
      int c;

      fscanf(arq_cena,"%s", nome_arq_objeto);

      strcpy(ant, dir_cena);
      arq_objeto = abre_arquivo(strcat(strcat(dir_cena,nome_arq_objeto),".pol"), "r");
      strcpy(dir_cena, ant);

      fscanf(arq_objeto,"%u %u\n", &n_vert, &n_polig);
      m->n_vert += n_vert;
      m->n_pol += n_polig;
      fclose(arq_objeto);

      while ((c=fgetc(arq_cena)) != '\n')
	; /* le ate o fim da linha */
    }
    le_material(arq_cena, &m->materiais[i]);
  }

  fseek(arq_cena, pos_arquivo, 0);

  m->vert = ALOCA(m->n_vert, ponto3d);
  m->polig = ALOCA(m->n_pol, poligono);
  matriz_transf = ALOCA(n_objetos_total, matriz4x4);

  offset_vert = offset_polig = ind_mat = 0;

  for(i = 0; i < n_malhas; i++)
  {
    unsigned int material = i;
    int c;

    fscanf(arq_cena,"%u\n", &n_objetos);
    for(j = 0; j < n_objetos; j++)
    {
      char nome_arq_objeto[TAM_NOME_ARQUIVO];

      fscanf(arq_cena,"%s", nome_arq_objeto);

      le_transf(arq_cena, matriz_transf[ind_mat]);
      le_objeto(nome_arq_objeto, m, material, matriz_transf[ind_mat++],
                &offset_vert, &offset_polig);
    }
    while ((c=fgetc(arq_cena)) != '\n')
      ;/* pula linha do material, ja foi lido */
  }

  m->normal = calcula_normal_vertices (m);
  /* free(matriz_transf); */
}

/*--------------------------------------------------------------*/
void le_cena(cena, nome_arq_cena)
t_cena *cena;
char *nome_arq_cena;
{

  FILE *arq_cena;
  char *env_cena,  ant[TAM_NOME_ARQUIVO];

  env_cena = getenv("CENA");
  strcpy(dir_cena, env_cena);
#ifdef DOS
  strcat(dir_cena, "\\");
#else
  strcat(dir_cena, "/");
#endif
  strcpy(ant, dir_cena);
  arq_cena = abre_arquivo(strcat(dir_cena,nome_arq_cena),"r");
  strcpy(dir_cena, ant);

  le_luzes(arq_cena, cena);

  le_camera(arq_cena, cena);

  le_malhas(arq_cena, cena);
}


/*-------------------------------------------------------------*/
ponto3d *le_normais (m, n_arq, nome_arq)
t_malha *m;
int n_arq;
char **nome_arq;
{
  int i, k;
  FILE **arq=ALOCA(n_arq, FILE *);


  m->normal = ALOCA (m->n_vert, ponto3d);
  i = 0;
  for(k = 0; k <  n_arq; k++)
  {
    arq[k] = abre_arquivo(nome_arq[k], "r");

    while(!feof(arq[k]))
    {
      fscanf (arq[k], "%f %f %f\n", &m->normal[i].x, &m->normal[i].y, &m->normal[i].z);
      i++;
    }
    fclose(arq[k]);
  }

  return m->normal;
}



void clipping_observador (m)
t_malha *m;
{
  int i, ind;

  ponto3d p_inter;
  int novo_indice;

  fila_vert *v_cabeca;
  fila_ind *i_cabeca;

  float d = distancia_plano ();

  v_cabeca = inicializa_fila_vertice (v_cabeca);

  /* Para todos os poligonos */

  for (i = 0; i < m->n_pol; i++) {
    poligono *p = &m->polig [i];
    int k, existe_dentro, existe_fora,
        j;
    
    /* Procurar um vertice que estaja dentro */
    for (k = 0, existe_dentro = 0; k < p->n_vert; k++) {
      if (m->vert [p->vert [k]].z < d) {
        existe_dentro = 1;
        break;
      }
    }

    if (!existe_dentro) {
      /* remover o poligono, pois esta
         completamente fora */
      remove_poligono (m, i--);
      continue;
    }
    /* Testa se o poligono todo esta dentro */
    for (ind = 0, existe_fora = 0; ind < p->n_vert; ind++) 
      if (m->vert [p->vert[ind]].z >= d) {
      /* possui ao menos um vertice fora */ 
        existe_fora = 1;
        break;
      }
    if (!existe_fora)
      continue; /* se nao possui fora entao vai para o prox polig */

    i_cabeca = inicializa_fila_indice (i_cabeca);
    /* Testa o caso especial em que o 1. vertice
       esta dentro e o ultimo esta fora */
    if (k == 0) {
      /* O primeiro vertice esta com certeza dentro */
      if (m->vert [p->vert [p->n_vert - 1]].z >= d) {
        calcula_intersecao (&m->vert [p->vert [p->n_vert - 1]],
                            &m->vert [p->vert [0]], d, &p_inter);
        novo_indice = insere_vertice (v_cabeca, m, &p_inter);
        insere_indice (i_cabeca, novo_indice);
      }
    }

    /* Anotar o indice do vertice k incondicionalmente,
       pois com certeza ele esta dentro */
    insere_indice (i_cabeca, p->vert[k]);

    /* Comecar do vertice seguinte a k e ir ate o final
       da lista, pois qualquer vertice anterior esta fora */
    for (j = k + 1; j < p->n_vert; j++) {
      if (m->vert [p->vert [j]].z < d)
        /* Esta dentro entao anote */
        insere_indice (i_cabeca, p->vert[j]);
      else {
        /* O vertice anterior esta fora */
        calcula_intersecao (&m->vert [p->vert [j]],
                            &m->vert [p->vert[j-1]], d, &p_inter);
        novo_indice = insere_vertice (v_cabeca, m, &p_inter);
        insere_indice (i_cabeca, novo_indice);
        /* ignora todos os vertices fora
           testar tambem se nao chegou ao final */
        while (j < p->n_vert  &&
               m->vert [p->vert [j]].z >= d) 
          j++;
        /* Se nao chegou ao final, entao esta
           em um vertice dentro */
        if (j < p->n_vert) {
          calcula_intersecao (&m->vert [p->vert [j]],
                              &m->vert [p->vert[j-1]], d, &p_inter);
          novo_indice = insere_vertice (v_cabeca, m, &p_inter);
          insere_indice (i_cabeca, novo_indice);
        }
      }
    }
    free (p->vert); 
    p->vert = ALOCA (i_cabeca->n_nodos, unsigned int);
    p->n_vert = i_cabeca->n_nodos; 
    for (ind = 0; i_cabeca->n_nodos > 0; ind++) 
      p->vert [ind] = remove_fila_indice (i_cabeca);
    desaloca_fila_indice (i_cabeca);
  }
  if (v_cabeca->n_nodos != 0) {
    ponto3d *temp;
    temp = (ponto3d *)realloc (m->vert, 
               (m->n_vert + v_cabeca->n_nodos) * sizeof (ponto3d));
    m->vert = temp;
    m->n_vert += v_cabeca->n_nodos;
    for (ind = m->n_vert - v_cabeca->n_nodos; v_cabeca->n_nodos > 0; ind++) 
      remove_fila_vertice (v_cabeca, &m->vert [ind]);
  }
  desaloca_fila_vertice (v_cabeca);
}

fila_vert *inicializa_fila_vertice (cabeca)
fila_vert *cabeca;
{
  cabeca = ALOCA (1, fila_vert);
  cabeca->n_nodos = 0;
  cabeca->fim = cabeca->inicio = ALOCA (1, nodo_vert);

  return cabeca;
}

int insere_vertice (cabeca, m, vertice)
fila_vert *cabeca;
t_malha *m;
ponto3d *vertice;
{
  cabeca->fim->prox = ALOCA (1, nodo_vert);
  cabeca->fim = cabeca->fim->prox;
  cabeca->n_nodos++;
  cabeca->fim->vert.x = vertice->x; 
  cabeca->fim->vert.y = vertice->y; 
  cabeca->fim->vert.z = vertice->z; 

  return (m->n_vert + cabeca->n_nodos - 1);
}

void remove_fila_vertice (cabeca, retorno)
fila_vert *cabeca;
ponto3d *retorno;
{
  nodo_vert *aux;

  retorno->x = cabeca->inicio->prox->vert.x;
  retorno->y = cabeca->inicio->prox->vert.y;
  retorno->z = cabeca->inicio->prox->vert.z;
  aux = cabeca->inicio->prox;
  cabeca->inicio->prox = aux->prox;
  free (aux); 
  cabeca->n_nodos--; 
}

fila_ind *inicializa_fila_indice (cabeca)
fila_ind *cabeca;
{
  cabeca = ALOCA (1, fila_ind);
  cabeca->n_nodos = 0;
  cabeca->fim = cabeca->inicio = ALOCA (1, nodo_ind);

  return cabeca;
}

void desaloca_fila_vertice (cabeca)
fila_vert *cabeca;
{
  free (cabeca->inicio);
  free (cabeca);
}

void insere_indice (cabeca, indice)
fila_ind *cabeca;
int indice;
{
  cabeca->fim->prox = ALOCA (1, nodo_ind);
  cabeca->fim = cabeca->fim->prox;
  cabeca->n_nodos++;
  cabeca->fim->indice = indice; 
}

int remove_fila_indice (cabeca)
fila_ind *cabeca;
{
  int retorno;
  nodo_ind *aux;

  retorno = cabeca->inicio->prox->indice;
  aux = cabeca->inicio->prox;
  cabeca->inicio->prox = aux->prox;
  free (aux); 
  cabeca->n_nodos--; 

  return retorno;
}

void desaloca_fila_indice (cabeca)
fila_ind *cabeca;
{
  free (cabeca->inicio);
  free (cabeca);
}
