/* malha.h */

#define RED   0
#define GREEN 1
#define BLUE  2

#define KA 0
#define KD 1
#define KS 2

#define LUZ_PONTUAL  1
#define LUZ_SPOT     2
#define LUZ_SOL      3

#define TAM_NOME_ARQUIVO 200

typedef float true_color[3];

typedef unsigned char RGB[3];

typedef struct tabela_materiais {
  char *nome;
  float coeficiente[3][3];
  unsigned int exp_phong;
  struct tabela_materiais *prox;
} tabela_materiais;

typedef struct vetor_materias {
  float coeficiente[3][3];
  unsigned int exp_phong;
} vetor_materiais;


typedef struct poligono {
  unsigned int n_vert, cor, *vert;
  int material;
} poligono;

typedef struct malha {
  int n_pol, n_vert;
  true_color *cor_vert;
  ponto3d *vert, *normal;
  poligono *polig;
  vetor_materiais *materiais;
} t_malha;

typedef struct normal_vertice {
  ponto3d vert;
  int n;
} normal_vertice;

typedef struct luz {
  float  intensidade[3];
  int     tipo;
  ponto3d posicao,
          direcao;
  float  coss;
} t_luz;

typedef struct camera {
  ponto3d   observador,
            direcao,
            normal,
            vertical;
  float    foco,
            esqj, infj, dirj, supj;
  int       tipo_proj;
} t_camera;


typedef struct cena{
  unsigned n_luzes;
  t_luz    *luzes;
  t_camera camera;
  t_malha  malha;
} t_cena;

typedef struct nodo_vert {
  ponto3d vert;
  struct nodo_vert *prox;
} nodo_vert;

typedef struct fila_vert {
  int n_nodos;
  nodo_vert *inicio, *fim; 
} fila_vert;

typedef struct nodo_ind {
  int indice;
  struct nodo_ind *prox;
} nodo_ind;

typedef struct fila_ind {
  int n_nodos;
  nodo_ind *inicio, *fim;
} fila_ind;

/* prototipos das funcoes */

FILE *abre_arquivo PROTO((char *, char *));

void le_luzes PROTO((FILE *a, t_cena *));

void le_camera PROTO((FILE *, t_cena *));

void le_transf PROTO((FILE *, matriz4x4));

void le_objeto PROTO((char *, t_malha *, unsigned int, matriz4x4,
                     unsigned int *, unsigned int *));

void le_material PROTO((FILE *, vetor_materiais *));


void le_malhas PROTO((FILE *, t_cena *));

void le_cena PROTO((t_cena *, char *));

ponto3d *le_normais PROTO((t_malha *, int, char **));

void clipping_observador PROTO ((t_malha *));

fila_vert *inicializa_fila_vertice PROTO ((fila_vert *));

int insere_vertice PROTO ((fila_vert *, t_malha *, ponto3d *));

void remove_fila_vertice PROTO ((fila_vert *, ponto3d *));

fila_ind *inicializa_fila_indice PROTO ((fila_ind *));

void desaloca_fila_vertice PROTO ((fila_vert *));

void insere_indice PROTO ((fila_ind *, int));

int remove_fila_indice PROTO ((fila_ind *));

void desaloca_fila_indice PROTO ((fila_ind *));

void calcula_normal PROTO((t_malha *m, poligono *p, ponto3d *normal));

void calcula_normais PROTO((t_malha *m));

ponto3d *calcula_normal_vertices PROTO((t_malha *m));

void remove_poligono PROTO((t_malha *, int));
