/*
 *  our.c
 *  Demonstrates how to get keyboard and mouse events in an canvas
 *  window.  Looks for keyboards, pointer movement and button
 *  events and displays the info in the canvas.
 */
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/xv_xrect.h>

#include "../bibliot/f.h"
#include "../bibliot/malha.h"
#include "../bibliot/camera.h"
#include "sl.h"

#include <stdlib.h>

t_cena *cena;
t_malha *malha;
XPoint *vert_tela;

t_cena *inicializa ();
cria_vert_tela()
{
  int i;

  for(i=0; i < malha->n_vert; i++)
  {
    ponto3d p;
    ponto2d r, s;
    
    muda_referencial(&malha->vert[i], &p);
    projecao(&p, &r);
    mapeia(&r, &s);
    vert_tela[i].x = s.x;
    vert_tela[i].y = s.y;
  }
}




/*
 * repaint_proc()
 *      Called to repaint the canvas in response to damage events
 *      and the initial painting of the canvas window.
 *      Displays the keyboard, pointer and button message strings
 *      after erasing the previous messages.
 */
void
repaint_proc(canvas, paint_window, dpy, xwin, xrects)
Canvas        canvas;           /* Ignored */
Xv_Window     paint_window;     /* Ignored */
Display      *dpy;
Window        xwin;
Xv_xrectlist *xrects;           /* Ignored */
{
    GC gc = DefaultGC(dpy, DefaultScreen(dpy));
    static int width, height;
    int i, j;
   
    width = (int)xv_get(paint_window, XV_WIDTH);
    height = (int)xv_get(paint_window, XV_HEIGHT); 
  
    cria_vert_tela(); 
     
    
    XClearWindow(dpy, xwin);
    for (i = 0; i < malha->n_pol; i++){
      poligono *pol = &malha->polig[i];
      for (j = 0; j < pol->n_vert-1; j++) { 
      XDrawLine(dpy, xwin, gc,          vert_tela[pol->vert[j]].x,
                               height - vert_tela[pol->vert[j]].y,
                                        vert_tela[pol->vert[j+1]].x,
                               height - vert_tela[pol->vert[j+1]].y);
      }                        
      XDrawLine(dpy, xwin, gc,           vert_tela[pol->vert[pol->n_vert-1]].x,
                                height - vert_tela[pol->vert[pol->n_vert-1]].y,
                                         vert_tela[pol->vert[0]].x, 
                                height - vert_tela[pol->vert[0]].y);
    }                                                            
}

/*
 * event_proc()
 *      Called when an event is received in the canvas window.
 *      Updates the keyboard, pointer and button message strings
 *      and then calls repaint_proc() to paint them to the window.
 */
void
event_proc(window, event)
Xv_Window window;
Event    *event;
{
   if (event_is_ascii (event)) 
     switch (event_action (event)) {
       case '-':
         afasta ();
         break;
       case '=':
         aproxima ();
         break;
       case '7':
         gira_orientacao ((double) 0.1);
         break;       
       case '1':
         gira_orientacao ((double) -0.1);
         break; 
       case '6':
         roda_horizontal ((double) 0.1);
         break;
       case '4':
         roda_horizontal ((double) -0.1);
         break;
       case '2':
         roda_vertical ((double) 0.1);
         break;
       case '8':
         roda_vertical ((double) -0.1);
         break;
       case 'l':
         direita_esquerda ((double) 0.1);
         break;
       case 'k':
         direita_esquerda ((double) -0.1);
         break;
       case 'q':
         sobe_desce ((double) 0.1);
         break;
       case 'a':
         sobe_desce ((double) -0.1);
         break;
       case ' ': {
   /*      printf (" Posicao: %lf %lf %lf\n", _posicao.x, _posicao.y, _posicao.z);
         printf (" Direcao: %lf %lf %lf\n", _direcao.x, _direcao.y, _direcao.z);
         printf (" Horizon: %lf %lf %lf\n", _vert
         break; */
     }
    }
   else
      switch (event_action(event)) {
          case ACTION_SELECT:
          case MS_LEFT:
              afasta ();
              afasta ();
              afasta ();
              break;
          case ACTION_ADJUST:
          case MS_MIDDLE:
              break;
          case ACTION_MENU:
          case MS_RIGHT:
              aproxima ();
              aproxima ();
              aproxima ();
              break;
          case WIN_RESIZE: {
              int width, height;
              
              width = (int)xv_get(window, XV_WIDTH);
              height = (int)xv_get(window, XV_HEIGHT); 
              tela (0,0,height, width);
              break;
          }
          case WIN_REPAINT:
              break;
          default:
              return;
      }
      
      repaint_proc ( (Canvas) NULL,
                      window,
                      (Display *) xv_get (window, XV_DISPLAY),
                      xv_get (window, XV_XID),
                      (Xv_xrectlist *) NULL);
}
 /*
 * main()
 *      Create a canvas specifying a repaint procedure.
 *      Get the paint window for the canvas and set the input
 *      mask and the event procedure.
 */
main(argc, argv)
int argc;
char *argv[];
{
    Frame       frame;
    Canvas      canvas; 
    int i;   

    if (argc != 2) {
      printf ("Our Arquivo_de_Cena\n");
      exit (0);
    }

    cena = inicializa (argv [1]);

       
    malha = &cena->malha;
    vert_tela = ALOCA(malha->n_vert, XPoint); 
    cria_vert_tela();

    /* Initialize XView */
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

    /* Create windows -- base frame and canvas. */
    frame = (Frame)xv_create(NULL, FRAME, NULL);

    canvas = (Canvas)xv_create(frame, CANVAS,
        XV_WIDTH,               500,
        XV_HEIGHT,              500,
        CANVAS_X_PAINT_WINDOW,  TRUE,
        CANVAS_REPAINT_PROC,    repaint_proc,
        NULL);
    window_fit(frame);

    /* Set input mask */
    xv_set(canvas_paint_window(canvas),
        WIN_EVENT_PROC,         event_proc,
        WIN_CONSUME_EVENTS,
        KBD_DONE, KBD_USE, LOC_DRAG, LOC_MOVE, LOC_WINENTER,
        LOC_WINEXIT, WIN_ASCII_EVENTS,
        WIN_RESIZE, WIN_MOUSE_BUTTONS,
        NULL,
        NULL);

    /* Start event loop */
    xv_main_loop(frame);
}
       

