/* sl.c */


#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#define DOS

#ifdef DOS
#include <alloc.h>
#include "f.h"
#include "malha.h"
#include "camera.h"
#else
#include <malloc.h>
#include "../bibliot/f.h"
#include "../bibliot/malha.h"
#include "../bibliot/camera.h"
#endif
#include "sl.h"

/* A luz pontual e calculada da seguinte forma:             */
/* Dadas a posicao da luz, o ponto e a normal naquele ponto.*/
/* Calcula-se: um vetor do ponto ate a luz,                 */
/*             um vetor do ponto ate o observador,          */
/* A intensidade da luz difusa e o cosseno entre o vetor    */
/* para a luz e a normal.                                   */
/* A intensidade da luz especular e o cosseno entre o vetor */
/* que indica a direcao de reflexao da luz e a normal.      */
void calc_luz_pontual(luz, p, normal, material, m, I)
t_luz *luz;
ponto3d *p;
ponto3d *normal;
int material;
t_malha *m;
float  I[];
{
  float difusa, especular;
  ponto3d luzv, obsv, bissv;

  subtracao(&luz->posicao, p, &luzv);   /* vetor para a luz */
  normalizacao(&luzv, &luzv);           /* normalizacao do vetor para a luz */
  difusa = produto_escalar(normal, &luzv); /* intensidade a luz difusa */
  if(difusa < 0.0)
    difusa = 0.0;
  if(difusa > 1.0)
    difusa = 1.0;
  I[0] += difusa * luz->intensidade[RED];  
  I[1] += difusa * luz->intensidade[GREEN];
  I[2] += difusa * luz->intensidade[BLUE];

  realXvetor(-1.0, p, &obsv);  /* vetor para o observador */
  normalizacao(&obsv, &obsv);  /* normalizacao do vetor para o observador */
  adicao(&obsv, &luzv, &bissv); /* calculo da bissetriz entre o vetor para o observador */
                                /* e o vetor para a luz. */
  normalizacao(&bissv, &bissv); /* normalizacao do vetor bissetriz */
  /* o vetor bissetriz indica a direcao de reflexao da luz */
  especular = produto_escalar(normal, &bissv); /* intensidade da luz especular, */
					       /* sem o expoente Phong     */
  if(especular < 0.0)
    especular = 0.0;
  if(especular > 1.0)
    especular = 1.0;
  especular = pow(especular, (float) m->materiais[material].exp_phong); /* intensidade */
                                                             /* da luz especular */
  I[3] += especular * luz->intensidade[RED];
  I[4] += especular * luz->intensidade[GREEN];
  I[5] += especular * luz->intensidade[BLUE];
}

/* O calculo da luz spot e identico ao da luz pontual, com */
/* a excecao de que ele so e realizado se o angulo entre o */
/* vetor para a luz e o vetor direcao da luz for menor que */
/* um dado angulo especificado.                            */
void calc_luz_spot(luz, p, normal, material, m, I)
t_luz * luz;
ponto3d *p;
ponto3d *normal;
int material;
t_malha *m;
float  I[];
{
  float difusa, especular, coss;
  ponto3d luzv, obsv, bissv;

  subtracao(&luz->posicao, p, &luzv);
  normalizacao(&luzv, &luzv);
  if ( (coss = produto_escalar(&luzv, &luz->direcao)) < 0.0 )
     return; /* ponto esta atraz da luz */
  if ( coss < luz->coss)
     return; /* luz nao incide sobre o ponto */

  difusa = produto_escalar(normal, &luzv);
  if(difusa < 0.0)
    difusa = 0.0;
  if(difusa > 1.0)
    difusa = 1.0;
  I[0] += difusa * luz->intensidade[RED];
  I[1] += difusa * luz->intensidade[GREEN];
  I[2] += difusa * luz->intensidade[BLUE];

  realXvetor(-1.0, p, &obsv);
  normalizacao(&obsv, &obsv);
  adicao(&obsv, &luzv, &bissv);
  normalizacao(&bissv, &bissv);
  especular = produto_escalar(normal, &bissv);
  if(especular < 0.0)
    especular = 0.0;
  if(especular > 1.0)
    especular = 1.0;
  especular = pow(especular,(float) m->materiais[material].exp_phong);
  I[3] += especular * luz->intensidade[RED];
  I[4] += especular * luz->intensidade[GREEN];
  I[5] += especular * luz->intensidade[BLUE];
}

/* Na luz do tipo sol o que muda e que a direcao de incidencia da */
/* luz e constante, como se a luz estivesse situada no infinito.  */
void calc_luz_sol(luz, p, normal, material, m, I)
t_luz * luz;
ponto3d *p;
ponto3d *normal;
int material;
t_malha *m;
float  I[];
{
  float difusa, especular;
  ponto3d obsv, obsn, bissv, bissn, menos_direcao, direcaon;


  normalizacao(&menos_direcao, &direcaon);
  difusa = produto_escalar(normal, &direcaon);
  if(difusa < 0.0)
    difusa = 0.0;
  if(difusa > 1.0)
    difusa = 1.0;
  I[0] += difusa * luz->intensidade[RED];
  I[1] += difusa * luz->intensidade[GREEN];
  I[2] += difusa * luz->intensidade[BLUE];

  realXvetor(-1.0, p, &obsv);
  normalizacao(&obsv, &obsn);
  adicao(&obsn, &direcaon, &bissv);
  normalizacao(&bissv, &bissn);
  especular = produto_escalar(normal, &bissn);
  if(especular < 0.0)
    especular = 0.0;
  if(especular > 1.0)
    especular = 1.0;
  especular = pow(especular, (float) m->materiais[material].exp_phong);
  I[3] += especular * luz->intensidade[RED];
  I[4] += especular * luz->intensidade[GREEN];
  I[5] += especular * luz->intensidade[BLUE];
}


extern int maxcolor;

void cor_ponto(cena, p, normal, polig, cor_rgb)
t_cena *cena;
ponto3d *p;
ponto3d *normal;
poligono *polig;
true_color cor_rgb;
{
  float cor, ambiente[3], I[6] ;
  int material = polig->material, i, j;
  t_malha *m = &cena->malha;
  t_luz *luz = &cena->luzes[0];
  
  ambiente[0] = luz->intensidade[RED];
  ambiente[1] = luz->intensidade[GREEN];
  ambiente[2] = luz->intensidade[BLUE];
  
  for(i = 0; i < 6; i++)
   I[i] = 0.0;

  for(i = 1; i < cena->n_luzes; i++) {
    luz = &cena->luzes[i];

    switch(luz->tipo) {
      case LUZ_PONTUAL:
        calc_luz_pontual(luz, p, normal, material, m, I);
        break;
      case LUZ_SPOT:
        calc_luz_spot(luz, p, normal, material, m, I);
        break;
      case LUZ_SOL:
	     calc_luz_sol(luz, p, normal, material, m, I);
        break;
    }
  }

  for(i = 0; i < 6; i++)
    if(I[i] > 1.0)
       I[i] = 1.0;

  cor = I[0]*m->materiais[material].coeficiente[RED][KD]
	+ I[3]*m->materiais[material].coeficiente[RED][KS]
        + ambiente[RED]*m->materiais[material].coeficiente[RED][KA];
  if (cor < 0.0)
    cor = 0.0;

  if (cor > 1.0)
    cor = 1.0;

  cor *= (float)(maxcolor - 1);
  cor_rgb[RED] = cor;

  cor = I[1]*m->materiais[material].coeficiente[GREEN][KD]
	+ I[4]*m->materiais[material].coeficiente[GREEN][KS]
        + ambiente[GREEN]*m->materiais[material].coeficiente[GREEN][KA];
  if (cor < 0.0)
    cor = 0.0;
  if (cor > 1.0)
    cor = 1.0;
  cor *= (float)(maxcolor - 1);
  cor_rgb[GREEN] = cor;

  cor = I[2]*m->materiais[material].coeficiente[BLUE][KD]
	+ I[5]*m->materiais[material].coeficiente[BLUE][KS]
        + ambiente[BLUE]*m->materiais[material].coeficiente[BLUE][KA];
  if (cor < 0.0)
    cor = 0.0;
  if (cor > 1.0)
    cor = 1.0;
  cor *= (float)(maxcolor - 1);
  cor_rgb[BLUE] = cor;
}

/*--------------------------------------------------------------*\
 | calc_cor_vert (cena)                                         |
 | Calcula a cor de cada vertice dos poligonos                  |
 |                                                              |
 |                                                              |
\*--------------------------------------------------------------*/
void calc_cor_vert(cena)
t_cena *cena;
{
  int i, j, *vert_pol;
  t_malha *m = &cena->malha;

  m->cor_vert = ALOCA(m->n_vert, true_color);
  vert_pol = ALOCA(m->n_vert, int);
  for (i = 0; i < m->n_vert; i++)
    vert_pol[i] = -1;
  for(i=0; i < m->n_pol; i++) {
     poligono *p = &m->polig[i];

     for(j=0; j < p->n_vert; j++)
       vert_pol[p->vert[j]] = i;
  }
  for(i=0; i < m->n_vert; i++)
     if (vert_pol[i] != -1)   
      cor_ponto(cena, &m->vert[i], &m->normal[i], &m->polig[vert_pol[i]],
	      m->cor_vert[i]);

  free(vert_pol); 
 }

/*--------------------------------------------------------------*/
/* cria_vert_base_observador.                                   */
/* Recebe: um ponteiro para malha.                              */
/* Retorna: nada.                                               */
/* Funcao: muda a base de todos os vertices para a base do      */
/* observador.                                                  */
/*--------------------------------------------------------------*/
void cria_vert_base_observador(m)
t_malha *m;
{
  int i;

  for(i=0; i < m->n_vert; i++) {
    muda_referencial(&m->vert[i], &m->vert[i]);
    muda_base(&m->normal[i], &m->normal[i]);
  }
}


/* -------------------------------------------------------------*/
/* vertice_proj *cria_vert_proj_gour(t_malha *m, vertice_proj *vert_proj) */
/* Recebe: um ponteiro para a malha, um ponteiro para o vetor de*/
/* vertices projetados que sera' criado.                        */
/* Retorna: o ponteiro para o vetor de vertices projetados.     */
/* Funcao: cria, a partir dos vertices da malha, ja em coorde-  */
/* nadas da base do observador, um vetor onde as coordenadas x  */
/* e y estao ja mapeadas na tela, e a coordenada z esta na base */
/* do observador.                                               */
/* -------------------------------------------------------------*/
vertice_proj *cria_vert_proj(m, vert_proj)
t_malha *m;
vertice_proj *vert_proj;
{
  int i;

  float d = distancia_plano ();
  
  for(i=0; i < m->n_vert; i++) {
    ponto2d r, s;

    if (m->vert[i].z > d)
       continue;
    projecao(&m->vert[i], &r);
    mapeia(&r, &s);
    vert_proj[i].x = s.x;
    vert_proj[i].y = s.y;
    vert_proj[i].z = m->vert[i].z;
  }
}

void muda_referencial_luzes(cena)
t_cena *cena;
{
  int i;

  for(i = 0; i < cena->n_luzes; i++) {
    t_luz *luz = &cena->luzes[i];

    switch(luz->tipo) {
      case LUZ_PONTUAL:
           muda_referencial(&luz->posicao, &luz->posicao);
           break;
      case LUZ_SPOT:
           muda_referencial(&luz->posicao, &luz->posicao);
           muda_base(&luz->direcao, &luz->direcao);
	   realXvetor(-1.0, &luz->direcao, &luz->direcao); /* inversao da direcao */
	   normalizacao(&luz->direcao, &luz->direcao); /* normalizacao da direcao */
	   /* a direcao e modificada para facilitar o calculo da luz */
           break;
      case LUZ_SOL:
           muda_base(&luz->direcao, &luz->direcao);
           break;
    }
  }
}


nodo_la **bucket_sort(bucket, m, vert_proj, n_linhas)
nodo_la **bucket;
t_malha *m;
vertice_proj *vert_proj;
int n_linhas;
{
  int i,j;

  cria_vert_proj(m, vert_proj);

  for (i = 0; i < n_linhas; i++)
     bucket[i] = NULL;

  for(i=0; i < m->n_pol; i++) {
     
    poligono *p = &m->polig[i];
    int ant;

    for(j=0, ant=p->n_vert-1; j < p->n_vert; ant=j++) {
       
      aresta a;

      a.v1 = p->vert[ant];
      a.v2 = p->vert[j];
      if(vert_proj[a.v1].y != vert_proj[a.v2].y) { 
	 /* aresta nao horizontal */
        int k = (vert_proj[a.v1].y < vert_proj[a.v2].y)? vert_proj[a.v1].y:
                                                       vert_proj[a.v2].y;
        
      
	     int vmenor = (vert_proj[a.v1].y < vert_proj[a.v2].y)? a.v1: a.v2;
        /* vmenor contem o indice do vertice com menor y */
        
        int vmaior = (vmenor == a.v1) ? a.v2 : a.v1;
        /* vmaior contem o indice do vertice com maior y */
        
        int esqt, inft, dirt, supt;
        /* coordenadas da tela :                          */
	/* esquerda, inferior, direita superior           */

        nodo_la *q = ALOCA(1, nodo_la);

        pega_tela (&esqt, &inft, &dirt, &supt); /* obtem as coordenadas da tela */

        /* Clipping referente a cena. Parte inferior e superior da tela. */
        
        /* esta primeira parte descarta as arestas que estao abaixo da tela e recalcula */
        /* o vertice das que cortam a parte inferior da tela.                           */
        
        if ((vert_proj[vmenor].y < inft && vert_proj[vmaior].y < inft) ||
          (vert_proj[vmaior].y > supt && vert_proj[vmenor].y > supt))
          continue;
        
        if (k < inft)
          k = inft;
        q->a.v1 = a.v1;
        q->a.v2 = a.v2;
        q->polig = i;
        q->prox = bucket[k];
        bucket[k] = q;          
           
      }
    }
  }
  return bucket;
}

void inicializa_z_buffer(z_buffer, tam)
float *z_buffer;
unsigned int tam;
{
  while(tam--)
    z_buffer[tam] = - INFINITO;
}

void insere_laa(laa, q)
nodo_laa *laa;
nodo_laa *q;
{
  nodo_laa *aux, *aux1;

  aux = laa->prox;
  aux1 = laa;
  while((aux != NULL) && (aux->polig < q->polig)) {
    aux1 = aux;
    aux = aux->prox;
  }
  while((aux != NULL) && (aux->polig == q->polig) && (aux->x < q->x)) {
    aux1 = aux;
    aux = aux->prox;
  }
  while((aux != NULL) && (aux->polig == q->polig) && (aux->x == q->x) && 
	      (aux->incrx < q->incrx)) {
    aux1 = aux;
    aux = aux->prox;
  }
  q->prox = aux;
  aux1->prox = q;
}

nodo_la *pega_nodo_gour(m, bucket, q, ycorr, vert_proj)
t_malha *m;
nodo_la *bucket;
nodo_laa *q;
unsigned int ycorr;
vertice_proj *vert_proj;
{
  nodo_la *aux = bucket;
  unsigned int v1,v2;
  float deltay, nlinhas;

  if(vert_proj[aux->a.v1].y < vert_proj[aux->a.v2].y) {
    v1 = aux->a.v1;
    v2 = aux->a.v2;
  }
  else {
    v1 = aux->a.v2;
    v2 = aux->a.v1;
  }
  
  /* Clipping referente a cena. */
  nlinhas = ycorr - vert_proj[v1].y;
       
  deltay = (float)(vert_proj[v2].y - vert_proj[v1].y);

  q->incrx = (float)(vert_proj[v2].x - vert_proj[v1].x)/deltay;
  q->x = (int)vert_proj[v1].x + q->incrx * nlinhas;


  q->incrz = (float)(vert_proj[v2].z - vert_proj[v1].z)/deltay;
  q->z = vert_proj[v1].z + q->incrz * nlinhas;

  q->incrcor[RED] = (float)(m->cor_vert[v2][RED] - m->cor_vert[v1][RED])/deltay;
  q->cor[RED] = m->cor_vert[v1][RED] + q->incrcor[RED] * nlinhas;

  q->incrcor[GREEN] = (float)(m->cor_vert[v2][GREEN] - m->cor_vert[v1][GREEN])/deltay;
  q->cor[GREEN] = m->cor_vert[v1][GREEN] + q->incrcor[GREEN] * nlinhas;

  q->incrcor[BLUE] = (float)(m->cor_vert[v2][BLUE] - m->cor_vert[v1][BLUE])/deltay;
  q->cor[BLUE] = m->cor_vert[v1][BLUE] + q->incrcor[BLUE] * nlinhas;

  q->delta_y = (int)(deltay - nlinhas);

  q->polig = aux->polig;

  bucket = aux->prox;
  free(aux); 

  return bucket;
}


nodo_la *pega_nodo_phong (m, bucket, q, ycorr, vert_proj)
t_malha *m;
nodo_la *bucket;
nodo_laa *q;
unsigned int ycorr;
vertice_proj *vert_proj;
{
  nodo_la *aux = bucket;
  unsigned int v1,v2;
  float nlinhas;
  float deltay;


  if(vert_proj[aux->a.v1].y < vert_proj[aux->a.v2].y) {
    v1 = aux->a.v1;
    v2 = aux->a.v2;
  }
  else {
    v1 = aux->a.v2;
    v2 = aux->a.v1;
  }

  /* Clipping referente a cena. */
  nlinhas = ycorr - vert_proj[v1].y;

  deltay = (float)(vert_proj[v2].y - vert_proj[v1].y);

  q->incrx = (float)(vert_proj[v2].x - vert_proj[v1].x)/deltay;
  q->x = (int)vert_proj[v1].x + q->incrx * nlinhas;

  q->incrz = (float)(vert_proj[v2].z - vert_proj[v1].z)/deltay;
  q->z = vert_proj[v1].z + q->incrz * nlinhas;

  q->incrcor[RED] = 0.0;
  q->cor[RED] = 0.0; 

  q->incrcor[GREEN] = 0.0;
  q->cor[GREEN] = 0.0;

  q->incrcor[BLUE] = 0.0;
  q->cor[BLUE] = 0.0;

  q->delta_y = (int)(deltay - nlinhas);
  
  q->polig = aux->polig;

  q->incrxm = (float)(m->vert[v2].x - m->vert[v1].x)/deltay;
  q->xm = m->vert[v1].x + q->incrxm * nlinhas;

  q->incrym = (float)(m->vert[v2].y - m->vert[v1].y)/deltay;
  q->ym = m->vert[v1].y + q->incrym * nlinhas;

  subtracao(&m->normal[v2], &m->normal[v1], &q->incrnormal);
  realXvetor(1/deltay, &q->incrnormal, &q->incrnormal);
  q->normal.x = m->normal[v1].x + (q->incrnormal).x * nlinhas;
  q->normal.y = m->normal[v1].y + (q->incrnormal).y * nlinhas;
  q->normal.z = m->normal[v1].z + (q->incrnormal).z * nlinhas;

  bucket = aux->prox;
  free(aux);
  return bucket;
}



void insere_nova_linha_laa_gour(m, bucket, laa, ycorr, vert_proj)
t_malha *m;
nodo_la *bucket;
nodo_laa *laa;
unsigned int ycorr;
vertice_proj *vert_proj;
{
  while(bucket != NULL) {
    nodo_laa *q = ALOCA(1, nodo_laa);

    bucket = pega_nodo_gour(m, bucket, q, ycorr, vert_proj);
    insere_laa(laa, q);
  }
}


void insere_nova_linha_laa_phong(m, bucket, laa, ycorr, vert_proj)
t_malha *m;
nodo_la *bucket;
nodo_laa *laa;
unsigned int ycorr;
vertice_proj *vert_proj;
{
  while(bucket != NULL) {
    nodo_laa *q = ALOCA(1, nodo_laa);

    bucket = pega_nodo_phong(m, bucket, q, ycorr, vert_proj);
    insere_laa(laa, q);
  }
}

FILE *arq; /* Ponteiro para o arquivo de saida. */

void traca_spans_gour(tamx, laa, z_buffer)
int tamx;
nodo_laa *laa;
float *z_buffer;
{
  RGB *linha; 
  nodo_laa *aux = laa->prox, *q;
  int i;

  linha = ALOCA(tamx, RGB);
  
  memset(linha, COR_BACKGROUND, tamx*sizeof(RGB));


  while(aux != NULL) {
    int x;
    float z, incz, cor[3], inccor[3], /* xm, incxm,
          ym, incym, */ deltax;
    nodo_laa *p, *q;

    p = aux;
    q = aux->prox;
    aux = q->prox;


   /* o span e' entre p e q */
    if((deltax = q->x - p->x) != 0) {

      int esqt, inft, dirt, supt; /* coordenadas da tela. */
      int fim;
      
      incz = (float)(q->z - p->z)/deltax;
      inccor[RED] = (q->cor[RED] - p->cor[RED])/(float)deltax;
      inccor[GREEN] = (q->cor[GREEN] - p->cor[GREEN])/(float)deltax;
      inccor[BLUE] = (q->cor[BLUE] - p->cor[BLUE])/(float)deltax;
     
      x=p->x;
      z=p->z;

      cor[RED] = p->cor[RED];
      cor[GREEN] = p->cor[GREEN];
      cor[BLUE] = p->cor[BLUE];

      /* Clipping referente a cena. */
      /* Esta parte calcula o clipping das arestas que estao */
      /* a esquerda e a direita da tela.                     */
      pega_tela (&esqt, &inft, &dirt, &supt);
      if (p->x < esqt)
        if (q->x > esqt){ /* a aresta corta a parte esquerda da tela */
	  float ncolunas = esqt - p->x;
	    
          z += incz * ncolunas;
          cor[RED] += inccor[RED] * ncolunas;
          cor[GREEN] += inccor[GREEN] * ncolunas;
          cor[BLUE] += inccor[BLUE] * ncolunas;
          x = esqt;
        } 
	else
          continue;
      
      /* clipping da parte direita da tela */
      fim = (q->x < dirt) ? q->x : dirt;
	          
      while(x < fim) {
        if(z > z_buffer[x]) {
          z_buffer[x] = z;
	  linha[x][RED] = (unsigned char)cor[RED];
          linha[x][GREEN] = (unsigned char)cor[GREEN];
          linha[x][BLUE] = (unsigned char)cor[BLUE];
        }

        x++;
        z += incz;
        cor[RED] += inccor[RED];
        cor[GREEN] += inccor[GREEN];
        cor[BLUE] += inccor[BLUE];
      }
    }
   }
   fwrite(linha, sizeof(RGB), tamx, arq);
   free(linha); 
}


void traca_spans_phong(cena, tamx, laa, z_buffer)
t_cena *cena;
int tamx;
nodo_laa *laa;
float *z_buffer;
{
  RGB *linha = ALOCA(tamx, RGB);
  nodo_laa *aux = laa->prox;
  int i;
  ponto3d normal_span, incnormal;
  z_buffer_otimizado *z_buffer_span = ALOCA(tamx, z_buffer_otimizado);

  
  memset(linha, COR_BACKGROUND, tamx*sizeof(RGB));
  memset(z_buffer_span, 0, tamx*sizeof(z_buffer_otimizado));
  
  while(aux != NULL) {
    int x;
    float z, incz, xm, incxm, ym, incym, deltax;
    nodo_laa *p, *q;

    p = aux;
    q = aux->prox;
    aux = q->prox;
    /* o span e' entre p e q */
    if((deltax = q->x - p->x) != 0.0) {
      int fim;
      int esqt, inft, dirt, supt;
      
      incz = (float)(q->z - p->z)/deltax;
      incxm = (float)(q->xm - p->xm)/deltax;
      incym = (float)(q->ym - p->ym)/deltax;
      subtracao(&q->normal, &p->normal, &incnormal);
      realXvetor(1/deltax, &incnormal, &incnormal);

      x = p->x;
      z = p->z;
      xm = p->xm;
      ym = p->ym;
      normal_span.x = p->normal.x;
      normal_span.y = p->normal.y;
      normal_span.z = p->normal.z;

      /* Clipping referente a cena. */
      /* Esta parte calcula o clipping das arestas que estao */
      /* a esquerda e a direita da tela.                     */
      pega_tela (&esqt, &inft, &dirt, &supt);
      if (p->x < esqt)
        if (q->x > esqt) { /* a aresta corta a parte esquerda da tela */
	  float ncolunas = esqt - p->x;
	    
          z += incz * ncolunas;
          xm += incxm * ncolunas;
          ym += incym * ncolunas;
          normal_span.x += incnormal.x * ncolunas;
          normal_span.y += incnormal.y * ncolunas;
          normal_span.z += incnormal.z * ncolunas;
          x = esqt;
       } 
       else
         continue;
      
      /* clipping da parte direita da tela */
      fim = (q->x < dirt) ? q->x : dirt;
	          
      while (x < fim) {
        ponto3d r;

	if (z > z_buffer[x]) {
          ponto3d normal_aux;
	  float cor[3];

    	  z_buffer[x] = z;
	  r.x = xm;
	  r.y = ym;
	  r.z = z;
          normal_aux.x = normal_span.x;
          normal_aux.y = normal_span.y;
	  normal_aux.z = normal_span.z;
	  normalizacao(&normal_aux, &normal_aux);
	  z_buffer_span[x].cena = cena;
	  z_buffer_span[x].r.x = r.x;
	  z_buffer_span[x].r.y = r.y;
	  z_buffer_span[x].r.z = r.z;
	  z_buffer_span[x].normal.x = normal_aux.x;
	  z_buffer_span[x].normal.y = normal_aux.y;
	  z_buffer_span[x].normal.z = normal_aux.z;
	  z_buffer_span[x].polig = &cena->malha.polig[p->polig];
	  
	 /* 	  cor_ponto(cena,&r,&normal_aux,&cena->malha.polig[p->polig],cor);
	  linha[x][RED] = (unsigned char)cor[RED];
          linha[x][GREEN] = (unsigned char)cor[GREEN];
          linha[x][BLUE] = (unsigned char)cor[BLUE];*/
	}
	x++;
	z += incz;
	xm += incxm;
	ym += incym;
	adicao(&normal_span, &incnormal, &normal_span);
      }
    }
  }
  for (i = 0; i < tamx; i++){
     if (z_buffer_span[i].cena != NULL) {
       cor_ponto(z_buffer_span[i].cena, &z_buffer_span[i].r,
	         &z_buffer_span[i].normal, z_buffer_span[i].polig,
	         z_buffer_span[i].cor);
       linha[i][RED] = (unsigned char)z_buffer_span[i].cor[RED];
       linha[i][GREEN] = (unsigned char)z_buffer_span[i].cor[GREEN];
       linha[i][BLUE] = (unsigned char)z_buffer_span[i].cor[BLUE];
     }
  }

  free(z_buffer_span);
  fwrite(linha, sizeof(RGB), tamx, arq);
  free(linha);
  
}

/* -----------------------------------------------*/
/*    Remove o elemento seguinte a p, da lista de arestas ativas.                                */
/*    Assume p->prox != NULL.                                                                                         */
void remove_proximo_laa(p)
nodo_laa *p;
{
  nodo_laa *aux = p->prox;

  p->prox = aux->prox;
  free(aux); 
}

void atualiza_laa_gour(laa)
nodo_laa *laa;
{
  nodo_laa *aux1 = laa->prox, *aux2 = laa;

  while(aux1 != NULL)
  {
    if(--aux1->delta_y <= 0)
    {
      remove_proximo_laa(aux2);
      aux1 = aux2->prox;
    }
    else
    {
      aux1->x += aux1->incrx;
      aux1->z += aux1->incrz;
      aux1->cor[RED] += aux1->incrcor[RED];
      aux1->cor[GREEN] += aux1->incrcor[GREEN];
      aux1->cor[BLUE] += aux1->incrcor[BLUE];
      aux1 = aux1->prox;
      aux2 = aux2->prox;
    }
  }

}


void atualiza_laa_phong(laa)
nodo_laa *laa;
{
  nodo_laa *aux1 = laa->prox, *aux2 = laa;

  while(aux1 != NULL)
  {
    if(--aux1->delta_y <= 0)
    {
       remove_proximo_laa(aux2);
      aux1 = aux2->prox;
   }
    else
    {
      aux1->x += aux1->incrx;
      aux1->z += aux1->incrz;
      aux1->cor[RED] += aux1->incrcor[RED];
      aux1->cor[GREEN] += aux1->incrcor[GREEN];
      aux1->cor[BLUE] += aux1->incrcor[BLUE];
      aux1->xm += aux1->incrxm;
      aux1->ym += aux1->incrym;
      adicao(&aux1->normal, &aux1->incrnormal, &aux1->normal);
      aux1 = aux1->prox;
      aux2 = aux2->prox;
    }
  }

}

void scan_line_gour(cena, tamx, tamy, nome_arq)
t_cena *cena;
unsigned int tamx;
unsigned int tamy;
char *nome_arq;
{

  int i;
  float *z_buffer = ALOCA(tamx, float);
  nodo_laa *laa = ALOCA(1, nodo_laa);
  nodo_la **bucket = ALOCA(tamy, nodo_la *);
  char nome_temp[20];
  vertice_proj *vert_proj;

  muda_referencial_luzes(cena); /* muda a luz para a base do observador. */
  cria_vert_base_observador(&cena->malha);
  clipping_observador (&cena->malha);
  calc_cor_vert(cena);

  vert_proj  = ALOCA(cena->malha.n_vert, vertice_proj);
  bucket = bucket_sort(bucket, &cena->malha, vert_proj, tamy);

  laa->prox = NULL;
  arq = abre_arquivo(strcat(nome_arq, ".rgb"),"wb");


  for(i=0; i < tamy; i++)
  {
    printf("\r                     \rLin:%i Mem:%lu",i, farcoreleft ());
    fflush(stdout);
    inicializa_z_buffer(z_buffer, tamx);
    insere_nova_linha_laa_gour(&cena->malha, bucket[i], laa, i, vert_proj);
    traca_spans_gour(tamx, laa, z_buffer);
    atualiza_laa_gour(laa);
  }
  fclose(arq);
  free(laa);
  free(vert_proj);
  free(z_buffer);
  free(bucket);
}



void scan_line_phong(cena, tamx, tamy, nome_arq)
t_cena *cena;
unsigned int tamx;
unsigned int tamy;
char *nome_arq;
{

  int i;
  nodo_laa *laa = ALOCA(1, nodo_laa);
  float *z_buffer = ALOCA(tamx, float);
  nodo_la **bucket = ALOCA(tamy, nodo_la *);
  vertice_proj *vert_proj;


  muda_referencial_luzes(cena); /* muda as luzes para a base do observador. */
  cria_vert_base_observador(&cena->malha);
  clipping_observador (&cena->malha);

  vert_proj  = ALOCA(cena->malha.n_vert, vertice_proj);
  bucket = bucket_sort(bucket, &cena->malha, vert_proj, tamy);

  laa->prox = NULL;

  arq = abre_arquivo(strcat(nome_arq, ".rgb"),"wb");

  for(i=0; i < tamy; i++)
  {
    printf("\r                     \rLin:%i Mem:%lu",i, farcoreleft ());
    fflush(stdout);
    inicializa_z_buffer(z_buffer, tamx);
    insere_nova_linha_laa_phong(&cena->malha, bucket[i], laa, i, vert_proj);
    traca_spans_phong(cena, tamx, laa, z_buffer);
    atualiza_laa_phong(laa);
  }
  fclose(arq);
  free(laa);
  free(vert_proj);
  free(z_buffer);
  free(bucket);
}


