 /* symtable.c - manusamento da tebela de simbolos */

#include <math.h>
#include <stdlib.h>

/*---------------------------------------------------------------------------*\
 | aloca_luz - Reserva memoria para um nodo de luz.                          |
 | recebe - Nada.                                                            |
 | retorna - O apontador do nodo alocado.                                    |
\*---------------------------------------------------------------------------*/
apont_luz aloca_luz ()
{
  apont_luz nodo_luz_aloc;
  int i;

  if ((nodo_luz_aloc = (apont_luz) malloc (sizeof (struct nodo_luz))) == NULL)
    sai_erro ("Erro: Nao consegui alocar memoria, impossivel prosseguir.\n");

  /* Inicializa as variaveis */

  nodo_luz_aloc->tipo_luz = 0;
  strcpy (nodo_luz_aloc->nome, "");
  nodo_luz_aloc->kd = nodo_luz_aloc->ks = nodo_luz_aloc->ka = 0.0;
  for (i = 0; i < 3; i++)
    nodo_luz_aloc->posicao [i] = nodo_luz_aloc->direcao [i] = 0.0;
  nodo_luz_aloc->abertura = 0.0;
  nodo_luz_aloc->prox_luz = NULL;

  return nodo_luz_aloc;
}

/*---------------------------------------------------------------------------*\
 | aloca_material - Reserva memoria para um nodo de material.                |
 | recebe - Nada.                                                            |
 | retorna - O apontador do nodo alocado.                                    |
\*---------------------------------------------------------------------------*/
apont_mater aloca_material ()
{
  apont_mater nodo_mater_aloc;
  int i;

  if ((nodo_mater_aloc = (apont_mater) malloc (sizeof (struct nodo_mater)))
       == NULL)
    sai_erro ("Erro: Nao consegui alocar memoria, impossivel prosseguir.\n");

  /* Inicializa as variaveis */

  strcpy (nodo_mater_aloc->nome, "");
  for (i = 0; i < 3; i++)
    nodo_mater_aloc->kd [i] = 0.0;
  for (i = 0; i < 3; i++)
    nodo_mater_aloc->ks [i] = 0.0;
  for (i = 0; i < 3; i++)
    nodo_mater_aloc->ka [i] = 0.0;
  nodo_mater_aloc->const_phong = 0.0;
  nodo_mater_aloc->prox_mater = NULL;
  nodo_mater_aloc->num_objetos = 0;

  return nodo_mater_aloc;
}

/*---------------------------------------------------------------------------*\
 | aloca_objeto - Reserva memoria para um nodo objeto.                       |
 | recebe - Nada.                                                            |
 | retorna - O apontador do nodo alocado.                                    |
\*---------------------------------------------------------------------------*/
apont_objeto aloca_objeto ()
{
  apont_objeto nodo_objeto_aloc;

  if ((nodo_objeto_aloc = (apont_objeto) malloc (sizeof (struct nodo_objeto)))
       == NULL)
    sai_erro ("Erro: Nao consegui alocar memoria, impossivel prosseguir.\n");

  /* Inicializa as variaveis */

  strcpy (nodo_objeto_aloc->nome, "");
  nodo_objeto_aloc->mater_objeto = NULL;
  nodo_objeto_aloc->ligac_objeto = NULL;
  nodo_objeto_aloc->prox_objeto = NULL;

  return nodo_objeto_aloc;
}

/*---------------------------------------------------------------------------*\
 | aloca_ligac - Reserva memoria para um nodo de ligacao.                    |
 | recebe - Nada.                                                            |
 | retorna - O apontador do nodo alocado.                                    |
\*---------------------------------------------------------------------------*/
apont_ligac aloca_ligac ()
{
  apont_ligac nodo_ligac_aloc;
  int i, j;

  if ((nodo_ligac_aloc = (apont_ligac) malloc (sizeof (struct nodo_ligac)))
       == NULL)
    sai_erro ("Erro: Nao consegui alocar memoria, impossivel prosseguir.\n");

  /* Inicializa as variaveis */

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      nodo_ligac_aloc->transf [i] [j] = 0.0;
  nodo_ligac_aloc->nivel_parent = 0;
  nodo_ligac_aloc->objeto_filho = NULL;
  nodo_ligac_aloc->prox_ligac = NULL;

  return nodo_ligac_aloc;
}

/*---------------------------------------------------------------------------*\
 | aloca_difinit - Reserva memoria para um nodo dos objetos definitivos.     |
 | recebe - Nada.                                                            |
 | retorna - O apontador do nodo alocado.                                    |
\*---------------------------------------------------------------------------*/
apont_definit aloca_definit ()
{
  apont_definit nodo_definit_aloc;
  int i, j;

  if ((nodo_definit_aloc = (apont_definit) malloc
      (sizeof (struct nodo_definit))) == NULL)
    sai_erro ("Erro: Nao consegui alocar memoria, impossivel prosseguir.\n");

  /* Inicializa as variaveis */

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      nodo_definit_aloc->transf [i] [j] = 0.0;
  strcpy (nodo_definit_aloc->nome, "");
  nodo_definit_aloc->material = NULL;
  nodo_definit_aloc->prox_definit = NULL;

  return nodo_definit_aloc;
}

/*---------------------------------------------------------------------------*\
 | insere_luz_lista - Coloca um nodo preenchido na lista de luzes            |
 | recebe - Um ponteiro para o nodo.                                         |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
insere_luz_lista (nodo_luz)
apont_luz nodo_luz;
{
  num_luzes++;
  nodo_luz->prox_luz = lista_luz;
  lista_luz = nodo_luz;
}

/*---------------------------------------------------------------------------*\
 | insere_mater_lista - Coloca um nodo preenchido na lista de materiais.     |
 | recebe - Um ponteiro para o nodo.                                         |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
insere_mater_lista (nodo_mater)
apont_mater nodo_mater;
{
  nodo_mater->prox_mater = lista_mater;
  lista_mater = nodo_mater;
}

/*---------------------------------------------------------------------------*\
 | insere_objeto_grafo - Coloca um nodo objeto na lista de objetos           |
 | recebe - Um ponterio para o objeto.                                       |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
insere_objeto_grafo (nodo_objeto)
apont_objeto nodo_objeto;
{
  if (grafo_objeto == NULL)
    grafo_fim_objeto = grafo_objeto = nodo_objeto;
  else {
    grafo_fim_objeto->prox_objeto = nodo_objeto;
    grafo_fim_objeto = nodo_objeto;
  }
}

/*---------------------------------------------------------------------------*\
 | insere_definit - Coloca um nodo objeto definitivo na lista.               |
 | recebe - Um ponterio para o objeto.                                       |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
insere_definit (nodo_objeto_definit)
apont_definit nodo_objeto_definit;
{
  nodo_objeto_definit->prox_definit = lista_definit;
  lista_definit = nodo_objeto_definit;
}

/*---------------------------------------------------------------------------*\
 | copia_matriz - Copia uma matriz para a outra.                             |
 | recebe - Duas matrizes 4x4 de floats.                                     |
 | retorna - Uma copia da segunda matriz.                                    |
\*---------------------------------------------------------------------------*/
void copia_matriz (matriz_a, matriz_b)
float matriz_a [4] [4];
float matriz_b [4] [4];
{
  int i, j;

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      matriz_a [i] [j] = matriz_b [i] [j];
}

/*---------------------------------------------------------------------------*\
 | liga_pai_filho - Insere um nodo ligacao na lista de ligacoes do nodo pai  |
 |                  para indicar qual e seu filho.                           |
 | recebe - Um poneiro para o pai e outro para o filho.                      |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
liga_pai_filho (nodo_pai, nodo_filho)
apont_objeto nodo_pai, nodo_filho;
{
  apont_ligac nodo_ligac;
  int i, j;

  nodo_ligac = aloca_ligac ();

  copia_matriz (nodo_ligac->transf, Y_matriz);
  nodo_ligac->nivel_parent = estado_parenteses;
  nodo_ligac->prox_ligac = nodo_pai->ligac_objeto;
  nodo_pai->ligac_objeto = nodo_ligac;
  nodo_ligac->objeto_filho = nodo_filho;
}

/*---------------------------------------------------------------------------*\
 | pesquisa_objeto - Pesquisa a existencia de um determinado objeto. A com-  |
 |                   paracao e feita pelo nome do objeto.                    |
 | recebe - O nome do objeto a ser procurado.                                |
 | retorna - Se o objeto exister retorna o endereco e retorna NULL caso con- |
 |           trario.                                                         |
\*---------------------------------------------------------------------------*/
apont_objeto pesquisa_objeto (nome_objeto)
char *nome_objeto;
{
  apont_objeto nodo_pesquisa;

  if (grafo_objeto == NULL)
    return NULL;

  nodo_pesquisa = grafo_objeto;
  while (nodo_pesquisa != NULL)
    if (!strcmp (nodo_pesquisa->nome, nome_objeto))
      return nodo_pesquisa;
    else
      nodo_pesquisa = nodo_pesquisa->prox_objeto;

  return NULL;
}

/*---------------------------------------------------------------------------*\
 | liga_objeto_material - Faz a ligacao do ponteiro de material do objeto    |
 |                        com o materia na lista dos materiais.              |
 | recebe - Um ponterio para o objeto e o nome do seu material.              |
 | retorna - Verdadeiro se a operacao foi bem sucedida e Falso caso contrario|
\*---------------------------------------------------------------------------*/
int liga_objeto_material (nodo_objeto, nome_material)
apont_objeto nodo_objeto;
char *nome_material;
{
  apont_mater nodo_pesquisa;

  if (lista_mater == NULL)
    return FALSO;

  nodo_pesquisa = lista_mater;
  while (nodo_pesquisa != NULL)
    if (!strcmp (nodo_pesquisa->nome, nome_material)) {
      nodo_objeto->mater_objeto = nodo_pesquisa;
      return VERDADEIRO;
    }
    else
      nodo_pesquisa = nodo_pesquisa->prox_mater;

  return FALSO;
}

/*---------------------------------------------------------------------------*\
 | multiplique_matriz - Multiplica uma matriz A pela B e coloca o resultado  |
 |                      na matriz A;                                         |
 | recebe - Duas matrizes 4x4 de floats.                                     |
 | retorna - A multiplicacao na primeira matriz.                             |
\*---------------------------------------------------------------------------*/
multiplique_matriz (matriz_a, matriz_b)
float matriz_a [4] [4], matriz_b [4] [4];
{
  float matriz_c [4] [4],
        result = 0.0;
  int i, j, k;

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++) {
      for (k = 0; k < 4; k++)
        result += matriz_a [i] [k] * matriz_b [k] [j];
      matriz_c [i] [j] = result;
      result = 0.0;
    }

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      matriz_a [i] [j] = matriz_c [i] [j];
}

/*---------------------------------------------------------------------------*\
 | atualiza_transformacoes - calcula as novas matrizes de transformacao dos  |
 |                           nodos delimitados pelos parenteses.             |
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
atualiza_transformacoes ()
{
  apont_ligac nodo_ligac_transf = objeto_temp->ligac_objeto;

  while (nodo_ligac_transf != NULL) {
    if (nodo_ligac_transf->nivel_parent == estado_parenteses + 1) {
      nodo_ligac_transf->nivel_parent--;
      multiplique_matriz (nodo_ligac_transf->transf, Y_matriz);
    }
    nodo_ligac_transf = nodo_ligac_transf->prox_ligac;
  }
}

/*---------------------------------------------------------------------------*\
 | desaloca_materiais - Desaloca a lista de materiais.                       |
 | recebe - O endereco do primeiro nodo da lista.                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
desaloca_materiais (nodo_desal_mater)
apont_mater nodo_desal_mater;
{
  if (nodo_desal_mater->prox_mater != NULL)
    desaloca_materiais (nodo_desal_mater->prox_mater);
  free (nodo_desal_mater);
}

/*---------------------------------------------------------------------------*\
 | desaloca_luzes - Desaloca a lista de luzes.                               |
 | recebe - O endereco do primeiro nodo da lista.                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
desaloca_luzes (nodo_desal_luz)
apont_luz nodo_desal_luz;
{
  if (nodo_desal_luz->prox_luz != NULL)
    desaloca_luzes (nodo_desal_luz->prox_luz);
  free (nodo_desal_luz);
}

/*---------------------------------------------------------------------------*\
 | desaloca_ligacoes - Desaloca lista de ligacoes de um nodo objeto.         |
 | recebe - Um ponteiro para o primeira ligacao do objeto.                   |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
desaloca_ligacoes (nodo_desal_ligac)
apont_ligac nodo_desal_ligac;
{
  if (nodo_desal_ligac->prox_ligac != NULL)
    desaloca_ligacoes (nodo_desal_ligac->prox_ligac);
  free (nodo_desal_ligac);
}

/*---------------------------------------------------------------------------*\
 | desaloca_definit - Desaloca lista de nodos definitivos.                   |
 | recebe - Um ponteiro para o primeiro nodo definitivo.                     |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
desaloca_definit (nodo_desal_definit)
apont_definit nodo_desal_definit;
{
  if (nodo_desal_definit->prox_definit != NULL)
    desaloca_definit (nodo_desal_definit->prox_definit);
  free (nodo_desal_definit);
}

/*---------------------------------------------------------------------------*\
 | desaloca_objetos - Desaloca a lista de objetos.                           |
 | recebe - Um ponteiro para o primeiro objetos da lista.                    |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
desaloca_objetos (nodo_desal_objeto)
apont_objeto nodo_desal_objeto;
{
  if (nodo_desal_objeto->prox_objeto != NULL)
    desaloca_objetos (nodo_desal_objeto->prox_objeto);
  if (nodo_desal_objeto->ligac_objeto != NULL)
    desaloca_ligacoes (nodo_desal_objeto->ligac_objeto);
  free (nodo_desal_objeto);
}

/*---------------------------------------------------------------------------*\
 | desaloca_memoria - Desaloca todas as lista e o grafo de objetos.          |
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
desaloca_memoria ()
{
  if (lista_mater != NULL)
    desaloca_materiais (lista_mater);

  if (lista_luz != NULL)
    desaloca_luzes (lista_luz);

  if (grafo_objeto != NULL)
    desaloca_objetos (grafo_objeto);

  if (lista_definit != NULL)
    desaloca_definit (lista_definit);
}

/*---------------------------------------------------------------------------*\
 | coloque_identidade_matriz - Anula a matriz transformacao com a identidade.|
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
coloque_identidade_matriz (matriz)
float matriz [4] [4];
{
  int i, j;

  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      matriz [i] [j] = 0.0;

  for (i = 0, j = 0; i < 4; i++, j++)
    matriz [i] [j] = 1.0;
}

/*---------------------------------------------------------------------------*\
 | grave_luzes - Grava as luzes utilizadas na cena.                          |
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
grave_luzes ()
{
  apont_luz nodo_luz_impr = lista_luz;

  fprintf (saida, "%d\n", num_luzes + 1);

  fprintf (saida, "%f %f %f\n", TS_luz_ambiente [0],
                                TS_luz_ambiente [1],
                                TS_luz_ambiente [2]);

  while (nodo_luz_impr != NULL) {
    fprintf (saida, "%f %f %f ", nodo_luz_impr->kd,
                                 nodo_luz_impr->ks,
				 nodo_luz_impr->ka);
    switch (nodo_luz_impr->tipo_luz) {
      case (S_SOL): {
        fprintf (saida, "3 "); /* indicacao para luz 'SOL'*/
        fprintf (saida, "%f %f %f\n", nodo_luz_impr->direcao [0],
                                      nodo_luz_impr->direcao [1],
                                      nodo_luz_impr->direcao [2]);
        break;
      }
      case (S_PONTUAL): {
        fprintf (saida, "1 "); /* indicacao para luz 'PONTUAL'*/
        fprintf (saida, "%f %f %f\n", nodo_luz_impr->posicao [0],
                                      nodo_luz_impr->posicao [1],
                                      nodo_luz_impr->posicao [2]);
        break;
      }
      case (S_SPOT): {
        fprintf (saida, "2 "); /* indicacao para luz 'SPOT'*/
        fprintf (saida, "%f %f %f ", nodo_luz_impr->posicao [0],
                                     nodo_luz_impr->posicao [1],
                                     nodo_luz_impr->posicao [2]);
        fprintf (saida, "%f %f %f ", nodo_luz_impr->direcao [0],
                                     nodo_luz_impr->direcao [1],
                                     nodo_luz_impr->direcao [2]);
        fprintf (saida, " %f\n", nodo_luz_impr->abertura);
        break;
      }
    }

    nodo_luz_impr = nodo_luz_impr->prox_luz;
  }
}

/*---------------------------------------------------------------------------*\
 | grave_camera - Grava as informacoes referentes a camera.                  |
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
grave_camera ()
{
  int foco;

  fprintf (saida, "%f %f %f\n", TS_observador [0],
                                TS_observador [1],
                                TS_observador [2]);
  fprintf (saida, "%f %f %f\n", TS_direcao [0],
                                TS_direcao [1],
                                TS_direcao [2]);
  if (TS_normal [0] == 0.0 &&
      TS_normal [1] == 0.0 &&
      TS_normal [2] == 0.0)     /* Normal do plano e igual a direcao */                         
    fprintf (saida, "%f %f %f\n", TS_direcao [0],
                                  TS_direcao [1],
                                  TS_direcao [2]);
  else                           /* Diferente */
    fprintf (saida, "%f %f %f\n", TS_normal [0],
                                  TS_normal [1],
                                  TS_normal [2]);
  fprintf (saida, "%f %f %f\n", TS_vertical [0],
                                TS_vertical [1],
                                TS_vertical [2]);
  foco = TS_foco;
  fprintf (saida, "%d\n", foco);
  fprintf (saida, "%f %f %f %f\n", TS_janela [0],
                                   TS_janela [1],
                                   TS_janela [2],
                                   TS_janela [3]);

  if (TS_projecao == TS_PARALELA)
    fprintf (saida, "1\n");
  else
    fprintf (saida, "2\n");
}

/*---------------------------------------------------------------------------*\
 | cria_lista - Cria a lista de nodos definitivos.                           |
 | recebe - O nodo do grafo para ser trabalhado.                             |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
cria_lista (nodo_trab, mater_trab, matriz_trab)
apont_objeto nodo_trab;
apont_mater mater_trab;
float matriz_trab [4] [4];
{
  apont_ligac ligacao_corrente = nodo_trab->ligac_objeto;
  apont_definit nodo_definit_preenc;
  float matriz_transf [4] [4];

  while (ligacao_corrente != NULL) {
    copia_matriz (matriz_transf, ligacao_corrente->transf);
    multiplique_matriz (matriz_transf, matriz_trab);
    if (mater_trab == NULL)
      cria_lista (ligacao_corrente->objeto_filho,
		  nodo_trab->mater_objeto,
		  matriz_transf);
    else
      cria_lista (ligacao_corrente->objeto_filho,
		  mater_trab,
		  matriz_transf);

    ligacao_corrente = ligacao_corrente->prox_ligac;
  }

  if (nodo_trab->ligac_objeto == NULL) { /* Nodo folha */
    insere_definit (nodo_definit_preenc = aloca_definit ());

    strcpy (nodo_definit_preenc->nome, nodo_trab->nome);
    copia_matriz (nodo_definit_preenc->transf, matriz_trab);
    nodo_definit_preenc->material = mater_trab;
  }
}

/*---------------------------------------------------------------------------*\
 | grafo_para_lista - Transforma os nodos do grafo em nodos definitivos em   |
 |                    uma lista.                                             |
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
grafo_para_lista ()
{
  apont_objeto objeto_procura;
  float matriz_chamada [4] [4];

  objeto_procura = grafo_objeto;
  while (objeto_procura != NULL) {
    if (!strcmp (objeto_procura->nome, NODOINICIO)) {
      break;
    }
    objeto_procura = objeto_procura->prox_objeto;
  }
  if (objeto_procura == NULL) {
    char mensagem [100];
    sprintf (mensagem, "Erro, nao existe nodo inicio: %s\n", NODOINICIO);
    sai_erro (mensagem);
  }

  coloque_identidade_matriz (matriz_chamada);

  cria_lista (objeto_procura,
	      objeto_procura->mater_objeto,
	      matriz_chamada);
}

/*---------------------------------------------------------------------------*\
 | grava_malha - Cria a parte referente ao objetos.                          |
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
grava_malha ()
{
  apont_definit nodo_definit_mater;
  apont_mater nodo_mater_malha;
  int total_malha = 0;


  nodo_definit_mater = lista_definit;
  while (nodo_definit_mater != NULL) {
    if (nodo_definit_mater->material == NULL) {
      char mensagem [100];
      sprintf (mensagem, "Erro, objeto %s sem material\n",
                          nodo_definit_mater->nome);
      sai_erro (mensagem);
    }
    nodo_definit_mater->material->num_objetos++;
    nodo_definit_mater = nodo_definit_mater->prox_definit;
  }

  nodo_mater_malha = lista_mater;
  while (nodo_mater_malha != NULL) {
    if (nodo_mater_malha->num_objetos != 0)
      total_malha++;
    nodo_mater_malha = nodo_mater_malha->prox_mater;
  }

  fprintf (saida, "%d\n", total_malha);
  nodo_mater_malha = lista_mater;
  while (nodo_mater_malha != NULL) {
    if (nodo_mater_malha->num_objetos != 0) {
      apont_definit nodo_procura;
      int phong;

      fprintf (saida, "%d\n", nodo_mater_malha->num_objetos);
      nodo_procura = lista_definit;
      while (nodo_procura != NULL) {
	   if (!strcmp (nodo_procura->material->nome,
		    nodo_mater_malha->nome)) {
          int i, j;
          fprintf (saida, "%s ", nodo_procura->nome);
          for (i = 0; i < 4; i++)
            for (j = 0; j < 4; j++)
              fprintf (saida, "%f ", nodo_procura->transf [i] [j]);
          fprintf (saida, "\n");
        }
        nodo_procura = nodo_procura->prox_definit;
      }
      phong = nodo_mater_malha->const_phong;
      fprintf (saida, "%f %f %f %f %f %f %f %f %f %d\n",
                               nodo_mater_malha->kd [0],
                               nodo_mater_malha->ks [0],
                               nodo_mater_malha->ka [0],
                               nodo_mater_malha->kd [1],
                               nodo_mater_malha->ks [1],
                               nodo_mater_malha->ka [1],
                               nodo_mater_malha->kd [2],
                               nodo_mater_malha->ks [2],
                               nodo_mater_malha->ka [2],
                               phong);
    }
    nodo_mater_malha = nodo_mater_malha->prox_mater;
  }
}

/*---------------------------------------------------------------------------*\
 | grave_arquivo - Cria o arquivo de saida.                                  |
 | recebe - Nada.                                                            |
 | retorna - Nada.                                                           |
\*---------------------------------------------------------------------------*/
grave_arquivo ()
{
  char *diretorio, nome_arq_completo [80];

  diretorio = getenv ("CENA");
  strcpy (nome_arq_completo, diretorio);

#ifdef DOS
  strcat (nome_arq_completo, "\\");
#else
  strcat (nome_arq_completo, "/");
#endif

  strcat (nome_arq_completo, ARQUIVO_SAIDA);
  if ((saida = fopen (nome_arq_completo, "w")) == NULL)
    sai_erro ("Erro abrindo arquivo de saida.\n");
  grave_luzes ();
  grave_camera ();
  grafo_para_lista ();
  grava_malha ();
  fclose (saida);
}

/*----------------------------------------------------------------------------*\
 | le_arquivo_entrada -  Abre os arquivos de entrada e temporario, chama a    |
 |                     inicializacao da tabela de simbolos e chama o parser   |
 | Recebe - A linha de comandos.                                              |
 | Retorna - Nada.                                                            |
\*----------------------------------------------------------------------------*/

le_arquivo_entrada (nome_arquivo_entrada)
char *nome_arquivo_entrada;
{
  char *diretorio, 
       nome_arq_completo [1000],
       nome_arq_pos_cpp  [1000],   /* Nome do arquivo a ser gerado pelo CPP */
       linha_de_comando  [1000];
  
  diretorio = getenv ("CENA");
  strcpy (nome_arq_completo, diretorio);
  
#ifdef DOS
  strcat (nome_arq_completo, "\\");
#else
  strcat (nome_arq_completo, "/");
#endif

  strcat (nome_arq_completo, nome_arquivo_entrada);

  /* Pre-processador da linguagem C */
  strcpy (nome_arq_pos_cpp, nome_arq_completo);
  strcat (nome_arq_pos_cpp, ".tmp");
  strcpy (linha_de_comando, "/lib/cpp -P ");
  strcat (linha_de_comando, nome_arq_completo);
  strcat (linha_de_comando, " >  ");
  strcat (linha_de_comando, nome_arq_pos_cpp);
  system (linha_de_comando);       /* cpp -P arquivo_cena > arquivo_entrada */

  if ((yyin = fopen (nome_arq_pos_cpp, "r")) == NULL) {
    printf ("Erro abrindo arquivo de entrada: %s\n", nome_arq_pos_cpp);
    exit (EXIT_FAILURE);
  }
  if ((yyout = fopen (ARQ_LIXO ,"w")) == NULL) {
    printf("Erro abrindo arquivo de saida.\n");
    exit(EXIT_FAILURE);
  }
  
  if(yyparse()!=0)
    printf("%d erros na leitura do arquivo de entrada.\n", Y_erros);

  grave_arquivo ();

  desaloca_memoria ();

  fclose (yyin);
  fclose (yyout);
  
  /* Deletar o arquivo temporario */
  strcpy (linha_de_comando, "rm ");
  strcat (linha_de_comando, nome_arq_pos_cpp);
  system (linha_de_comando);
  
  /* Deletar o arquivo criado pelo lex */
  strcpy (linha_de_comando, "rm ");
  strcat (linha_de_comando, ARQ_LIXO);
  system (linha_de_comando);
  
}

/*----------------------------------------------------------------------------*\
 | sair_erro - Aborta o programa na ocorrencia de erros irrecuperaveis.       |
 | Recebe - Uma mensagem.                                                     |
 | Retorna - Nada.                                                            |
\*----------------------------------------------------------------------------*/
sai_erro (s)
char *s;
{
  printf ("Linha - %d\n", Y_linha);
  printf ("%s\n", s);
  desaloca_memoria ();
  exit (EXIT_FAILURE);
}

/*----------------------------------------------------------------------------*\
 | yyerror - Funcao chamada pelo parser quando da ocorrencia de um erro.      |
 | Recebe - Uma mensagem.                                                     |
 | Retorna - Nada.                                                            |
\*----------------------------------------------------------------------------*/
yyerror (s)
char *s;
{
  Y_erros++;
  printf ("%s\n", s);
  if (Y_erros > 50) {
    printf ("Erro fatal: muitos erros.\n");
    exit(EXIT_FAILURE);
  }
}

/*----------------------------------------------------------------------------*\
 | yywrap - Recuperacao de erros do yacc.                                     |
 | Recebe - Nada.                                                             |
 | Retorna - Nada.                                                            |
\*----------------------------------------------------------------------------*/
yywrap ()
{
  return -1;
}
